package com.gree.mall.contest.config.aop;

import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.plus.service.UserService;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

@Aspect
@Slf4j
public class ApiAspect {

    @Value("${spring.profiles.active}")
    private String profiles;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    UserService userService;

    @Pointcut("@annotation(io.swagger.v3.oas.annotations.Operation)")
    public void auth() {}

    @Before("auth()")
    public void doBefore(JoinPoint joinPoint) throws RemoteServiceException {
            start(joinPoint);
    }

    private void start(JoinPoint joinPoint) throws RemoteServiceException {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        Method sourceMethod = getSourceMethod(joinPoint);
        HttpServletRequest request = attributes.getRequest();
        String servletPath = request.getServletPath();
        ApiNotAuth apiNotAuth = sourceMethod.getAnnotation(ApiNotAuth.class);
        if(apiNotAuth != null || request.getRequestURI().contains("api-docs")){
            return;
        }

        if(!profiles.equals("prd")){
            if(request.getHeader(Constant.TOKEN_NAME) != null && request.getHeader(Constant.TOKEN_NAME).equals("1111")){
                return;
            }
            if(request.getParameter(Constant.TOKEN_NAME) != null && request.getParameter(Constant.TOKEN_NAME).equals("1111")){
                return;
            }
        }
        String userId = CommonUtils.getUserId(request);
        if (StringUtils.isBlank(userId)) {
            throw new RemoteServiceException(ResponseHelper.ResponseCode_AUTH_ERROR, "请求不合法");
        }
        //taijpcapi=pc  zfminiapp = 小程序接口
        String issuer = CommonUtils.getIssuer(request);
        if(!servletPath.contains(issuer)){
            throw new RemoteServiceException("暂无权限访问，请联系相关人员");
        }
        //如果是师傅，状态无效，无法访问
//        String appid = CommonUtils.getAPPID();
//        if(StringUtils.isNotBlank(appid)) {
//            AdminCompanyWechat adminCompanyWechat = (AdminCompanyWechat) redisUtil.get(Constant.RedisPrefix.CLIENT + appid);
//            if (adminCompanyWechat == null) {
//                throw new RemoteServiceException("非法请求");
//            }
//            if (adminCompanyWechat.getFlag() == 2) {
//                User user = userService.getById(userId);
//                //还未授权手机号的不用检查
//                if (StringUtils.isBlank(user.getMobile())) {
//                    return;
//                }
//                Integer count = workerService.lambdaQuery().eq(Worker::getUserId, userId).eq(Worker::getStatus, true).count();
//                if (count == 0) {
//                    throw new RemoteServiceException("您暂未入驻任何网点");
//                }
//            }
//        }

    }

    private Method getSourceMethod(JoinPoint jp) {
        Method proxyMethod = ((MethodSignature) jp.getSignature()).getMethod();
        try {
            return jp.getTarget().getClass().getMethod(proxyMethod.getName(), proxyMethod.getParameterTypes());
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        }
        return null;
    }

}
